package netmap

import (
	"bytes"
	"slices"
	"testing"

	"github.com/stretchr/testify/require"
)

func TestNodeInfo_Clone(t *testing.T) {
	var ni NodeInfo
	ni.publicKey = []byte{2}
	attr := Attribute{
		key:     "key",
		value:   "value",
		parents: []string{"parent", "parent2"},
	}
	ni.attributes = []Attribute{attr}
	ni.addresses = []string{"5", "6"}

	c := ni.Clone()

	require.True(t, c != &ni)
	require.True(t, bytes.Equal(c.publicKey, ni.publicKey))
	require.True(t, &(c.publicKey[0]) != &(ni.publicKey[0]))
	require.True(t, &(c.attributes[0]) != &(ni.attributes[0]))
	require.True(t, slices.Compare(c.addresses, ni.addresses) == 0)
	require.True(t, &(c.addresses[0]) != &(ni.addresses[0]))
}

func TestAttribute_Clone(t *testing.T) {
	attr := Attribute{
		key:     "key",
		value:   "value",
		parents: []string{"parent1", "parent2"},
	}

	c := attr.Clone()

	require.True(t, c != &attr)
	require.True(t, c.key == attr.key)
	require.True(t, &(c.key) != &(attr.key))
	require.True(t, &(c.value) != &(attr.value))
	require.True(t, c.value == attr.value)
	require.True(t, &(c.parents[0]) != &(attr.parents[0]))
	require.True(t, slices.Compare(c.parents, attr.parents) == 0)
}
